@@ -12,111 +12,33 @@
 #include "android/inputstreamthread_p.h"
 #include "android/lowenergynotificationhub_p.h"
 
+QT_BEGIN_NAMESPACE
+
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
 
-typedef QHash<QByteArray, QJniObject> JCachedStringFields;
+typedef QHash<QByteArray, QString> JCachedStringFields;
 Q_GLOBAL_STATIC(JCachedStringFields, cachedStringFields)
-
-//Java field names
-static const char * const javaActionAclConnected = "ACTION_ACL_CONNECTED";
-static const char * const javaActionAclDisconnected = "ACTION_ACL_DISCONNECTED";
-static const char * const javaActionBondStateChanged = "ACTION_BOND_STATE_CHANGED";
-static const char * const javaActionDiscoveryStarted = "ACTION_DISCOVERY_STARTED";
-static const char * const javaActionDiscoveryFinished = "ACTION_DISCOVERY_FINISHED";
-static const char * const javaActionFound = "ACTION_FOUND";
-static const char * const javaActionScanModeChanged = "ACTION_SCAN_MODE_CHANGED";
-static const char * const javaActionUuid = "ACTION_UUID";
-static const char * const javaExtraBondState = "EXTRA_BOND_STATE";
-static const char * const javaExtraDevice = "EXTRA_DEVICE";
-static const char * const javaExtraPairingKey = "EXTRA_PAIRING_KEY";
-static const char * const javaExtraPairingVariant = "EXTRA_PAIRING_VARIANT";
-static const char * const javaExtraRssi = "EXTRA_RSSI";
-static const char * const javaExtraScanMode = "EXTRA_SCAN_MODE";
-static const char * const javaExtraUuid = "EXTRA_UUID";
+Q_GLOBAL_STATIC(QMutex, stringCacheMutex);
 
 /*
  * This function operates on the assumption that each
  * field is of type java/lang/String.
  */
-inline QByteArray classNameForStaticField(JavaNames javaName)
+QString valueFromStaticFieldCache(const char *key, const char *className, const char *fieldName)
 {
-    switch (javaName) {
-    case JavaNames::BluetoothAdapter: {
-        constexpr auto cn = QtJniTypes::className<QtJniTypes::BluetoothAdapter>();
-        return QByteArray(cn.data(), cn.size());
-    }
-    case JavaNames::BluetoothDevice: {
-        constexpr auto cn = QtJniTypes::className<QtJniTypes::BluetoothDevice>();
-        return QByteArray(cn.data(), cn.size());
-    }
-    default:
-        break;
-    }
-    return {};
-}
-
-QJniObject valueForStaticField(JavaNames javaName, JavaNames javaFieldName)
-{
-    //construct key
-    //the switch statements are used to reduce the number of duplicated strings
-    //in the library
-
-    const char *fieldName;
-    switch (javaFieldName) {
-    case JavaNames::ActionAclConnected:
-        fieldName = javaActionAclConnected; break;
-    case JavaNames::ActionAclDisconnected:
-        fieldName = javaActionAclDisconnected; break;
-    case JavaNames::ActionBondStateChanged:
-        fieldName = javaActionBondStateChanged; break;
-    case JavaNames::ActionDiscoveryStarted:
-        fieldName = javaActionDiscoveryStarted; break;
-    case JavaNames::ActionDiscoveryFinished:
-        fieldName = javaActionDiscoveryFinished; break;
-    case JavaNames::ActionFound:
-        fieldName = javaActionFound; break;
-    case JavaNames::ActionScanModeChanged:
-        fieldName = javaActionScanModeChanged; break;
-    case JavaNames::ActionUuid:
-        fieldName = javaActionUuid; break;
-    case JavaNames::ExtraBondState:
-        fieldName = javaExtraBondState; break;
-    case JavaNames::ExtraDevice:
-        fieldName = javaExtraDevice; break;
-    case JavaNames::ExtraPairingKey:
-        fieldName = javaExtraPairingKey; break;
-    case JavaNames::ExtraPairingVariant:
-        fieldName = javaExtraPairingVariant; break;
-    case JavaNames::ExtraRssi:
-        fieldName = javaExtraRssi; break;
-    case JavaNames::ExtraScanMode:
-        fieldName = javaExtraScanMode; break;
-    case JavaNames::ExtraUuid:
-        fieldName = javaExtraUuid; break;
-    default:
-        qCWarning(QT_BT_ANDROID) << "Unknown java field name passed to valueForStaticField():" << javaFieldName;
-        return QJniObject();
-    }
-
-    const QByteArray className = classNameForStaticField(javaName);
-    if (className.isEmpty()) {
-        qCWarning(QT_BT_ANDROID) << "Unknown java class name passed to valueForStaticField():" << javaName;
-        return QJniObject();
-    }
-
-    const QByteArray key = className + fieldName;
+    QMutexLocker lock(stringCacheMutex());
     JCachedStringFields::iterator it = cachedStringFields()->find(key);
     if (it == cachedStringFields()->end()) {
         QJniEnvironment env;
         QJniObject fieldValue = QJniObject::getStaticObjectField(
                                             className, fieldName, "Ljava/lang/String;");
         if (!fieldValue.isValid()) {
-            cachedStringFields()->insert(key, QJniObject());
-            return QJniObject();
+            cachedStringFields()->insert(key, {});
+            return {};
         }
-
-        cachedStringFields()->insert(key, fieldValue);
-        return fieldValue;
+        const QString string = fieldValue.toString();
+        cachedStringFields()->insert(key, string);
+        return string;
     } else {
         return it.value();
     }
@@ -126,7 +48,7 @@ void QtBroadcastReceiver_jniOnReceive(JNIEnv *env, jobject /*javaObject*/,
                                       jlong qtObject, QtJniTypes::Context context,
                                       QtJniTypes::Intent intent)
 {
-    reinterpret_cast<AndroidBroadcastReceiver*>(qtObject)->onReceive(env, context, intent);
+    reinterpret_cast<AndroidBroadcastReceiver*>(qtObject)->onReceive(env, context.object(), intent.object());
 }
 Q_DECLARE_JNI_NATIVE_METHOD(QtBroadcastReceiver_jniOnReceive, jniOnReceive)
 
@@ -140,7 +62,7 @@ Q_DECLARE_JNI_NATIVE_METHOD(QtBluetoothSocketServer_errorOccurred, errorOccurred
 static void QtBluetoothSocketServer_newSocket(JNIEnv */*env*/, jobject /*javaObject*/,
                                               jlong qtObject, QtJniTypes::BluetoothSocket socket)
 {
-    reinterpret_cast<ServerAcceptanceThread*>(qtObject)->javaNewSocket(socket);
+    reinterpret_cast<ServerAcceptanceThread*>(qtObject)->javaNewSocket(socket.object());
 }
 Q_DECLARE_JNI_NATIVE_METHOD(QtBluetoothSocketServer_newSocket, newSocket)
 
@@ -166,7 +88,7 @@ void QtBluetoothLE_leScanResult(JNIEnv *env, jobject, jlong qtObject,
         return;
 
     reinterpret_cast<AndroidBroadcastReceiver*>(qtObject)->onReceiveLeScan(
-                                                                env, bluetoothDevice, rssi,
+                                                                env, bluetoothDevice.object(), rssi,
                                                                 scanRecord);
 }
 Q_DECLARE_JNI_NATIVE_METHOD(QtBluetoothLE_leScanResult, leScanResult)
@@ -174,13 +96,13 @@ Q_DECLARE_JNI_NATIVE_METHOD(QtBluetoothLE_leScanResult, leScanResult)
 static const char logTag[] = "QtBluetooth";
 static const char classErrorMsg[] = "Can't find class \"%s\"";
 
-#define FIND_AND_CHECK_CLASS(CLASS_NAME)                             \
-clazz = env.findClass<CLASS_NAME>();                                 \
-if (!clazz) {                                                        \
-    __android_log_print(ANDROID_LOG_FATAL, logTag, classErrorMsg,    \
-                        QtJniTypes::className<CLASS_NAME>().data()); \
-    return JNI_FALSE;                                                \
-}                                                                    \
+#define FIND_AND_CHECK_CLASS(CLASS_NAME)                                     \
+clazz = env.findClass<CLASS_NAME>();                                         \
+if (!clazz) {                                                                \
+    __android_log_print(ANDROID_LOG_FATAL, logTag, classErrorMsg,            \
+                        QtJniTypes::Traits<CLASS_NAME>::className().data()); \
+    return JNI_FALSE;                                                        \
+}                                                                            \
 
 #define LEHUB_SCOPED_METHOD(Method) Q_JNI_NATIVE_SCOPED_METHOD(Method, LowEnergyNotificationHub)
 
@@ -213,7 +135,8 @@ static bool registerNatives()
                                    LEHUB_SCOPED_METHOD(lowEnergy_characteristicWritten),
                                    LEHUB_SCOPED_METHOD(lowEnergy_descriptorWritten),
                                    LEHUB_SCOPED_METHOD(lowEnergy_characteristicChanged),
-                                   LEHUB_SCOPED_METHOD(lowEnergy_serviceError)
+                                   LEHUB_SCOPED_METHOD(lowEnergy_serviceError),
+                                   LEHUB_SCOPED_METHOD(lowEnergy_remoteRssiRead)
                                }))
     {
         __android_log_print(ANDROID_LOG_FATAL, logTag,
@@ -263,6 +186,8 @@ static bool registerNatives()
     return true;
 }
 
+QT_END_NAMESPACE
+
 Q_BLUETOOTH_EXPORT jint JNICALL JNI_OnLoad(JavaVM* vm, void* /*reserved*/)
 {
     static bool initialized = false;
@@ -283,6 +208,9 @@ Q_BLUETOOTH_EXPORT jint JNICALL JNI_OnLoad(JavaVM* vm, void* /*reserved*/)
         return -1;
     }
 
+    const auto context = QNativeInterface::QAndroidApplication::context();
+    QtJniTypes::QtBtBroadcastReceiver::callStaticMethod<void>("setContext", context);
+
     if (!registerNatives()) {
         __android_log_print(ANDROID_LOG_FATAL, logTag, "registerNatives failed");
         return -1;
